/**
 *
 */
package org.jp.illg.dstar.util;

import java.util.Arrays;
import java.util.regex.Pattern;

import org.jp.illg.dstar.DStarDefines;

import lombok.NonNull;

public class DStarUtils {

	private static final Pattern callsignCharPattern =
		Pattern.compile("[^0-9A-Z/]");


	private DStarUtils() {super();}

	public static byte[] getNullAMBE(){
		return DStarDefines.VoiceSegmentNullBytes;
	}

	public static byte[] getEndAMBE() {
		return DStarDefines.VoiceSegmentNullBytes;
	}

	public static byte[] getLastAMBE() {
		return DStarDefines.VoiceSegmentLastBytes;
	}

	public static byte[] getEndSlowdata() {
		return DStarDefines.SlowdataEndBytes;
	}

	public static byte[] getLastSlowdata() {
		return DStarDefines.SlowdataLastBytes;
	}

	public static int calcCRC(byte[] data, int length) {
		return DStarCRC.calcCRC(data, length);
	}

	public static boolean isValidCallsignFullLength(String... callsigns) {
		return isValidCallsignLength(false, callsigns);
	}

	public static boolean isValidCallsignFullLength(char[]... callsigns) {
		return isValidCallsignLength(false, callsigns);
	}

	public static boolean isValidCallsignShortLegth(String... callsigns) {
		return isValidCallsignLength(true, callsigns);
	}

	public static boolean isValidCallsignShortLegth(char[]... callsigns) {
		return isValidCallsignLength(true, callsigns);
	}

	private static boolean isValidCallsignLength(boolean shortLength, String... callsigns) {
		if(callsigns == null) {return false;}

		for(String callsign : callsigns) {
			if(callsign == null) {return false;}

			if(
				(shortLength && callsign.length() != DStarDefines.CallsignShortLength) ||
				(!shortLength && callsign.length() != DStarDefines.CallsignFullLength)
			) {return false;}
		}

		return true;
	}

	private static boolean isValidCallsignLength(boolean shortLength, char[]... callsigns) {
		if(callsigns == null) {return false;}

		for(char[] callsign : callsigns) {
			if(callsign == null) {return false;}

			if(
				(shortLength && callsign.length != DStarDefines.CallsignShortLength) ||
				(!shortLength && callsign.length != DStarDefines.CallsignFullLength)
			) {return false;}
		}

		return true;
	}

	public static String formatFullCallsign(String callsign, char module){
		if(callsign == null){callsign = "";}

		char[] formattedCallsign = new char[DStarDefines.CallsignFullLength];
		Arrays.fill(formattedCallsign, ' ');

		int callsignLength = callsign.length() > 7 ? 7 : callsign.length();

		for(int index = 0; index < callsignLength; index++)
			formattedCallsign[index] = callsign.charAt(index);

		formattedCallsign[DStarDefines.CallsignFullLength - 1] = module;

		return String.valueOf(formattedCallsign);
	}

	public static String formatFullCallsign(String callsign){
		return formatFullCallsign(callsign, ' ');
	}

	public static String formatFullLengthCallsign(String callsign) {
		final String format = "%-" + DStarDefines.CallsignFullLength + "S";

		if(callsign == null) {callsign = "";}
		return String.format(format, callsign);
	}

	public static String formatShortLengthCallsign(String callsign) {
		final String format = "%-" + DStarDefines.CallsignShortLength + "S";

		if(callsign == null) {callsign = "";}
		return String.format(format, callsign);
	}

	public static String formatLengthShortMessage(String shortMessage) {
		final String format = "%-" + DStarDefines.DvShortMessageLength + "s";

		if(shortMessage == null) {shortMessage = "";}

		if(shortMessage.length() > DStarDefines.DvShortMessageLength)
			shortMessage = shortMessage.substring(0, DStarDefines.DvShortMessageLength);

		return String.format(format, shortMessage);
	}

	public static String replaceCallsignUnderbarToSpace(String callsign) {
		if(callsign != null)
			return callsign.replaceAll("_", " ");
		else
			return "";
	}

	public static String replaceCallsignSpaceToUnderbar(String callsign) {
		if(callsign != null)
			return callsign.replaceAll(" ", "_");
		else
			return "";
	}

	public static boolean writeInt32BigEndian(byte[] dstBuffer, int dstBufferOffset, int value32) {
		if(dstBuffer == null || dstBuffer.length < (dstBufferOffset + 4))
			return false;

		int value = value32;

		for(int c = 0; c < 4 && dstBuffer.length > (dstBufferOffset + c); c++) {
			dstBuffer[(dstBufferOffset + 3) - c] = (byte)value;
			value = value >> 8;
		}

		return true;
	}

	public static boolean writeInt32BigEndian(byte[] dstBuffer, int dstBufferOffset, long value32) {
		if(dstBuffer == null || dstBuffer.length < (dstBufferOffset + 4))
			return false;

		long value = value32;

		for(int c = 0; c < 4 && dstBuffer.length > (dstBufferOffset + c); c++) {
			dstBuffer[(dstBufferOffset + 3) - c] = (byte)value;
			value = value >> 8;
		}

		return true;
	}

	public static boolean writeBooleanBigEndian(byte[] dstBuffer, int dstBufferOffset, boolean bool) {
		return writeInt32BigEndian(dstBuffer, dstBufferOffset, bool ? 1 : 0);
	}

	public static boolean writeFullCallsignToBuffer(byte[] dstBuffer, int dstBufferOffset, char[] callsign) {
		if(dstBuffer == null || dstBufferOffset < 0 || callsign == null) {return false;}

		int copyLength = callsign.length;
		if(copyLength > DStarDefines.CallsignFullLength) {copyLength = DStarDefines.CallsignFullLength;}

		int overflow = (dstBufferOffset + copyLength) - dstBuffer.length;
		if(overflow > 0) {copyLength -= overflow;}

		for(int i = 0; i < copyLength; i++)
			dstBuffer[i + dstBufferOffset] = (byte)callsign[i];

		return true;
	}

	public static String reformatCallsign(@NonNull final String callsign) {
		String newCallsign =
			callsignCharPattern.matcher(callsign).replaceAll(" ");

		return newCallsign;
	}
}
